/*=========================================================================

Program:   Visualization Toolkit
Module:    vtkGRIB2Reader.cxx

Copyright (c) Xunlei Wu
All rights reserved.
See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

This software is distributed WITHOUT ANY WARRANTY; without even
the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the above copyright notice for more information.

=========================================================================*/
#include "vtkGRIB2Reader.h"

#include "vtkSmartPointer.h"
#include "vtkDataSetMapper.h"
#include "vtkImageData.h"
#include "vtkPointData.h"
#include "vtkTable.h"
#include "vtkInformation.h"
#include "vtkInformationVector.h"
#include "vtkObjectFactory.h"
#include "vtkActor.h"
#include "vtkRenderer.h"
#include "vtkRenderWindow.h"
#include "vtkRenderWindowInteractor.h"
#include "vtkStreamingDemandDrivenPipeline.h"
#include "vtkErrorCode.h"
#include "vtkStructuredGrid.h"

#ifdef _WIN32
#define _CRT_SECURE_NO_WARNINGS
#endif//_WIN32

vtkCxxRevisionMacro(vtkGRIB2Reader, "$Revision: 1.0 $");
vtkStandardNewMacro(vtkGRIB2Reader);

void vtkGRIB2Reader::PrintSelf(ostream& os, vtkIndent indent)
{
	this->Superclass::PrintSelf(os,indent);

	if (this->FileName){
		os << indent << "FileName: " << this->FileName << endl;
	}

	os << indent << "Dimensions: " << this->Dimensions[0] << ", "
		<< this->Dimensions[1] << ", " << this->Dimensions[2] << endl;

	os << indent << "Extents: " << this->Extents[0] << ", "
		<< this->Extents[1] << ", " << this->Extents[2] << ", "
		<< this->Extents[3] << ", " << this->Extents[4] << ", "
		<< this->Extents[5] << endl;

	os << indent << "Spacing: " << this->Spacing[0] << ", "
		<< this->Spacing[1] << ", " << this->Spacing[2] << endl;

	os << indent << "Origin: " << this->Origin[0] << ", "
		<< this->Origin[1] << ", " << this->Origin[2] << endl;

	os << indent << "TimeStep: " << this->TimeStep << endl;
}


vtkGRIB2Reader::vtkGRIB2Reader()
{
	vtkStructuredGrid *output = vtkStructuredGrid::New();
	this->SetOutput(output);
	// Releasing data for pipeline parallism.
	// Filters will know it is empty. 
	output->ReleaseData();
	output->Delete();

	this->FileName = NULL;
	this->SetNumberOfInputPorts(0);//0 for source or reader
	this->SetNumberOfOutputPorts(1);


	this->Dimensions[0] = -VTK_LARGE_INTEGER;
	this->Dimensions[1] = -VTK_LARGE_INTEGER;
	this->Dimensions[2] = -VTK_LARGE_INTEGER;

	this->Extents[0] = -VTK_LARGE_INTEGER;
	this->Extents[1] =  VTK_LARGE_INTEGER;
	this->Extents[2] = -VTK_LARGE_INTEGER;
	this->Extents[3] =  VTK_LARGE_INTEGER;
	this->Extents[4] = -VTK_LARGE_INTEGER;
	this->Extents[5] =  VTK_LARGE_INTEGER;

	this->Spacing[0] = -VTK_DOUBLE_MAX;
	this->Spacing[1] = -VTK_DOUBLE_MAX;
	this->Spacing[2] = -VTK_DOUBLE_MAX;

	this->Origin[0] = -VTK_DOUBLE_MAX;
	this->Origin[1] = -VTK_DOUBLE_MAX;
	this->Origin[2] = -VTK_DOUBLE_MAX;
}


vtkGRIB2Reader::~vtkGRIB2Reader()
{
	if(this->FileName){
		this->SetFileName(0);
	}
}


///The implementation of RequestInformation tells the VTK pipeline information about 
///the data before creating the output. In the case of image algorithms, this means 
///populating the extent, spacing and origin flags.
int vtkGRIB2Reader::RequestInformation(	vtkInformation* vtkNotUsed(request), 
										vtkInformationVector** vtkNotUsed(inputVector), 
										vtkInformationVector* outputVector)
{
	//----turn on DebugMacro----
	unsigned char preDebugFlag = this->Debug;
	if(!preDebugFlag)
		DebugOn();

	//fabricate data for debugging
	vtkInformation* outInfo = outputVector->GetInformationObject(0);

	//fabricate some values
	this->Extents[0] = 0;
	this->Extents[1] = 400;
	this->Extents[2] = 0;
	this->Extents[3] = 300;
	this->Extents[4] = 0;
	this->Extents[5] = 0;
	outInfo->Set(vtkStreamingDemandDrivenPipeline::WHOLE_EXTENT(), this->Extents, 6);
	
	//----restore Debug state----
	if(!preDebugFlag)
		DebugOff();
	
	return 1;
}
