/*=========================================================================

Program:   Visualization Toolkit
Module:    vtkGRIB2Reader.h

Copyright (c) Xunlei Wu
All rights reserved.
See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

This software is distributed WITHOUT ANY WARRANTY; without even
the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the above copyright notice for more information.

=========================================================================*/
// .NAME vtkGRIB2Reader - read GRIB Edition 2 files
// .SECTION Description
// vtkGRIB2Reader is a source object that reads GRIB2 files.

#ifndef __vtkGRIB2Reader_h__
#define __vtkGRIB2Reader_h__
#include "vtkStructuredGridAlgorithm.h"

//---- MACRO definitions ----
#ifndef PRINT_LOG_MACROS
#define PRINT_LOG_MACROS
#include <cstdio>//stdio.h
#include <iostream>
#include <sstream>
#define LOG_VERBOSE "(+)"
#define LOG_WARNING "(-)"
#define LOG_ERROR   "(X)"
#define CoutLog(LOG_TYPE) \
	std::cout << LOG_TYPE << " " << logHeader
#define pCoutLog(LOG_TYPE, rank) \
	std::cout << "Processor " << rank << ": " << LOG_TYPE << " " << logHeader
//#define PrintLog(LOG_TYPE, STRING) \
//	fprintf(stderr, "%s %s %s\n", LOG_TYPE, logHeader, STRING);
#endif//PRINT_LOG_MACROS

//#define ENABLE_DEBUG_VIEWER
#undef ENABLE_DEBUG_VIEWER

//subclass **Algorithm in order to build plugin rather than building into the source tree.
//class VTK_EXPORT vtkGRIB2Reader : public vtkStructuredGridReader
class VTK_EXPORT vtkGRIB2Reader : public vtkStructuredGridAlgorithm
{
public:
	static vtkGRIB2Reader *New();
	vtkTypeRevisionMacro(vtkGRIB2Reader, vtkStructuredGridAlgorithm);
	void PrintSelf(ostream& os, vtkIndent indent);

	vtkSetStringMacro(FileName);
	vtkGetStringMacro(FileName);

protected:
	vtkGRIB2Reader();
	~vtkGRIB2Reader();

	char* FileName;
	int Extents[6];
	int Dimensions[3];
	double Spacing[3], Origin[3];
	double TimeStep;

	//reimplementing base class virtual functions
	virtual int RequestInformation(vtkInformation*, vtkInformationVector**, vtkInformationVector*);
	//vitural function created by current class

private:
	vtkGRIB2Reader(const vtkGRIB2Reader&);
	void operator = (const vtkGRIB2Reader&);
};

#endif//__vtkGRIB2Reader_h__
